// CoMET - The Crimson Fields Map Editing Tool
// Copyright (C) 2002-2003 Jens Granseuer
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

////////////////////////////////////////////////////////////////////////
// edwidget.cpp
//
// History:
//  21-01-2003 - created
////////////////////////////////////////////////////////////////////////

#include "edwidget.h"
#include "misc.h"

////////////////////////////////////////////////////////////////////////
// NAME       : NumberWidget::ValidateKey
// DESCRIPTION: Only accept numbers for input.
// PARAMETERS : str - string currently entered in widget (not used)
//              key - char to be entered
//              pos - position at which to enter the char
// RETURNS    : TRUE if key is accepted, FALSE if refused
//
// HISTORY
////////////////////////////////////////////////////////////////////////

bool NumberWidget::ValidateKey( const char *str, unsigned short key,
                                unsigned short pos ) const {
  return ((key >= '1') && (key <= '9')) || ((key == '0') && (pos != 0));
}


#define GFX_CYCLE_W 7
#define GFX_CYCLE_H 12
#define GFX_CYCLE_X 138
#define GFX_CYCLE_Y 46

////////////////////////////////////////////////////////////////////////
// NAME       : CycleWidget::CycleWidget
// DESCRIPTION: Create a new cycle widget.
// PARAMETERS : id     - widget identifier
//              x      - left edge of widget relative to window border
//              y      - top edge of widget
//              w      - widget width
//              h      - widget height
//              key    - key equivalent for mouse clicks
//              flags  - see widget.h for details (default title
//                       placement is WIDGET_ALIGN_LEFT)
//              title  - widget title (may be NULL)
//              defval - default value, starting with 0
//              labels - array of available choices, terminated by a
//                       NULL-string
//              window - widget parent window
// RETURNS    : -
//
// HISTORY
////////////////////////////////////////////////////////////////////////

CycleWidget::CycleWidget( short id, short x, short y, unsigned short w,
             unsigned short h, short key, unsigned short flags,
             const char *title, unsigned short defval,
             const char *labels[], Window *window ) :
     ButtonWidget( id, x, y, w, h, key, flags, title, window ),
     maxval(0), choices(labels) {

  if ( !(flags & (WIDGET_ALIGN_CENTER|WIDGET_ALIGN_LEFT|
                  WIDGET_ALIGN_RIGHT|WIDGET_ALIGN_ABOVE)) )
    this->flags |= WIDGET_ALIGN_LEFT;

  image[0] = new Image( window->GetView()->GetSystemIcons(),
                        GFX_CYCLE_X, GFX_CYCLE_Y,
                        GFX_CYCLE_W, GFX_CYCLE_H );

  for ( int i = 1; labels[i]; ++i ) ++maxval;
  val = MIN( defval, maxval );
}

////////////////////////////////////////////////////////////////////////
// NAME       : CycleWidget::Draw
// DESCRIPTION: Draw the cycle widget.
// PARAMETERS : -
// RETURNS    : -
//
// HISTORY
////////////////////////////////////////////////////////////////////////

void CycleWidget::Draw( void ) {
  ButtonWidget::Draw();

  surface->FillRect( x + w - image[0]->Width() - 8, y + 4, 1, h - 8, Color(CF_COLOR_SHADOW) );
  surface->FillRect( x + w - image[0]->Width() - 7, y + 4, 1, h - 8, Color(CF_COLOR_HIGHLIGHT) );

  image[0]->Draw( surface, x + w - image[0]->Width() - 4, y + (h - image[0]->Height())/2 );

  // draw current label
  font->Write( choices[val], surface,
               x + (w - image[0]->Width() - 10 - font->TextWidth(choices[val]))/2,
               y + (h - font->Height()) / 2, *this );
}

////////////////////////////////////////////////////////////////////////
// NAME       : CycleWidget::Release
// DESCRIPTION: Update the button label and redraw the widget.
// PARAMETERS : -
// RETURNS    : -
//
// HISTORY
////////////////////////////////////////////////////////////////////////

void CycleWidget::Release( void ) {
  val = (val + 1) % (maxval + 1);

  ButtonWidget::Release();
}

